# -*- encoding: utf-8 -*-
"""Prints the location of python object definition in your file-system.
"""


from __future__ import print_function
import sys
import argparse
import importlib
import inspect
import re
import os
from textwrap import dedent
import ast
from collections import namedtuple

# =============== #
# Compute version #
# =============== #

__version__ = '0.2.0'
__revision__ = 'c506f3c3cd26033d1919c47d8463829a29ea8458'

_VERSION_SCRIPT = os.path.join(os.path.dirname(__file__),
                               "script", "version")

def get_version():
    if __version__ != 'dev':
        return __version__
    import subprocess as sp
    cmd = [_VERSION_SCRIPT, "get"]
    return sp.check_output(cmd).decode().strip()

def get_revision():
    if __revision__ != 'git':
        return __revision__
    import subprocess as sp
    cmd = [_VERSION_SCRIPT, "revision"]
    return sp.check_output(cmd).decode().strip()

# ===== #
# PyLoc #
# ===== #

class PylocError(Exception):
    """Base class of all exception raised by this module."""

class ModuleNameError(PylocError):

    def __init__(self, name, error):
        self.name = name
        self.error = error

    def __str__(self):
        return "failed to import '{}' ({}: {})"\
            .format(self.name,
                    type(self.error).__name__,
                    self.error)

class AttributeNameError(PylocError):

    def __init__(self, prefix, name):
        self.prefix = prefix
        self.name = name

    def __str__(self):
        return "cannot get attribute '%s' from '%s'" %(self.name, self.prefix)

Location = namedtuple('Location', 'filename line column')

class _ClassDefVisitor(ast.NodeVisitor):

    def __init__(self, qualname):
        self.qualname = qualname
        self.candidates = []
        self.path = []

    def visit_ClassDef(self, node):
        self.path.append(node)
        qualname = ".".join(n.name for n in self.path)
        if qualname == self.qualname:
            self.candidates.append(node)
        retval = self.generic_visit(node)
        self.path.pop()
        return retval

    def visit_FunctionDef(self, node):
        # Do not descend into FunctionDef
        pass

def _get_file_content(filename):
    with open(filename) as f:
        return f.read()

def _search_classdef(filename, qualname):
    source = _get_file_content(filename)
    root_node = ast.parse(source, filename)
    visitor = _ClassDefVisitor(qualname)
    visitor.visit(root_node)
    return visitor.candidates

def _iter_class_methods(obj):
    for attr in dir(obj):
        val = getattr(obj, attr)
        if inspect.isfunction(val) or inspect.ismethod(val):
            yield val

def _get_line(obj):
    if inspect.ismethod(obj):
        obj = obj.__func__
    if inspect.isfunction(obj):
        obj = obj.__code__
    if inspect.istraceback(obj):
        obj = obj.tb_frame
    if inspect.isframe(obj):
        obj = obj.f_code
    if inspect.iscode(obj) and hasattr(obj, 'co_firstlineno'):
        return obj.co_firstlineno
    return None

def _disamb_class_loc(candidates, obj):
    methods = list(_iter_class_methods(obj))
    if not methods:
        return
    meth_line = min(_get_line(m) for m in methods)
    best_candidate = None
    best_dist = None
    # Select the closest candidates coming before the first method definition
    for c in candidates:
        if c.lineno < meth_line: # Must come before
            dist = meth_line - c.lineno
            if best_dist is None or best_dist > dist:
                best_dist = dist
                best_candidate = c
    return best_candidate

def _candidate_nodes_to_locations(filename, candidates):
    return sorted([Location(filename, c.lineno, c.col_offset)
                   for c in candidates])

def _get_node_name(node):
    if hasattr(node, "name"):
        if hasattr(node, "asname") and node.asname:
            return node.asname
        return node.name
    elif hasattr(node, "id"):
        return node.id
    else:
        raise ValueError("do not know how to get name of node: {!r}"
                         .format(node))

def _iter_assigned_names(node):
    assert isinstance(node, ast.Assign)
    for target in node.targets:
        for n in ast.walk(target):
            if isinstance(n, ast.Name):
                yield n

class _AssignVisitor(ast.NodeVisitor):

    def __init__(self, qualname):
        self.qualname = qualname
        self.candidates = []
        self.path = []

    def visit_ClassDef(self, node):
        self.path.append(node)
        retval = self.generic_visit(node)
        self.path.pop()
        return retval

    def visit_Assign(self, node):
        for name_node in _iter_assigned_names(node):
            qualname = ".".join(_get_node_name(n)
                                for n in self.path+[name_node])
            if qualname == self.qualname:
                self.candidates.append(node)

    def visit_ImportFrom(self, node):
        for name_node in node.names:
            qualname = ".".join(_get_node_name(n)
                                for n in self.path+[name_node])
            if qualname == self.qualname:
                self.candidates.append(node)

    def visit_FunctionDef(self, node):
        # Do not descend into FunctionDef
        pass

def _search_assign(filename, qualname):
    source = _get_file_content(filename)
    root_node = ast.parse(source, filename)
    visitor = _AssignVisitor(qualname)
    visitor.visit(root_node)
    return visitor.candidates

def _is_inspectable(obj):
    return inspect.isclass(obj) \
        or inspect.ismethod(obj) \
        or inspect.isfunction(obj) \
        or inspect.ismodule(obj)

def _find_frozen_file(obj, qualname, filename):
    mo = re.match(r"^<frozen (.*)>$", filename)
    if mo:
        try:
            mod = importlib.import_module(mo.group(1))
        except ImportError:
            pass
        else:
            filename = mod.__file__
    return filename

def _find_file_harder(obj, qualname, filename):
    strategies = (_find_frozen_file,)
    i = 0
    while True:
        if os.path.exists(filename):
            return filename
        if i >= len(strategies):
            raise RuntimeError("failed to get an existing source file name")
        strategy = strategies[i]
        filename = strategy(obj, qualname, filename)
        i += 1

def _get_locations(obj, qualname):
    filename = inspect.getsourcefile(obj)
    if not filename:
        return [Location(inspect.getfile(obj), None, None)]
    filename = _find_file_harder(obj, qualname, filename)
    if inspect.ismodule(obj):
        return [Location(filename, None, None)]
    if inspect.isclass(obj):
        ### Search for ClassDef node in AST.
        candidates = _search_classdef(filename, qualname)
        if candidates:
            if len(candidates) > 1:
                # Try to disambiguite by locating the method defined in the
                # class.
                candidate = _disamb_class_loc(candidates, obj)
                if candidate is not None:
                    return [Location(filename,
                                     candidate.lineno,
                                     candidate.col_offset)]
            return _candidate_nodes_to_locations(filename, candidates)
        ### Search for Assign node in AST
        candidates = _search_assign(filename, qualname)
        if candidates:
            return _candidate_nodes_to_locations(filename, candidates)
        return [Location(filename, None, None)]
    return [Location(filename, _get_line(obj), None)]

def _has_same_filename(locs):
    filename = locs[0].filename
    return all(map(lambda x: x.filename == filename, locs))

def _from_pydoc_format(target):
    """
    The pydoc target format has no column to separate the package/module part
    from the object part.
    """
    parts = target.split(".")
    nparts = len(parts)
    if nparts <= 1:
        return target
    for i in range(nparts, 0, -1):
        mod_part = ".".join(parts[:i])
        try:
            importlib.import_module(mod_part)
        except ImportError:
            pass
        else:
            qual_part = ".".join(parts[i:])
            if qual_part:
                return mod_part + ':' + qual_part
            return mod_part
    # We cannot import the target at all. Return it as is. The rest of the
    # program will report the error.
    return target

def pyloc(target):
    """Return possible location defining ``target`` object.

    ``target`` named "module[:qualname]".

    Return a list of location namedtuple where the first value is
    the filename, the second the line number and the third the column number.
    The line and column number may be None if no applicable (i.e. for module
    or package) or if they cannot be found.

    Inspired by 'inspect._main()' and 'inspect.findsource()' by
      Ka-Ping Yee <ping@lfw.org> and
      Yury Selivanov <yselivanov@sprymix.com>
    """
    if not target:
        raise ValueError("target must be a non-empty string")
    if ":" not in target:
        target = _from_pydoc_format(target)
    mod_name, has_qualname, qualname = target.partition(":")
    ### Try to import the module containing the given target.
    try:
        module = importlib.import_module(mod_name)
    except ImportError as exc:
        raise ModuleNameError(mod_name, exc)
    ### Get location of module
    if not has_qualname:
        return _get_locations(module, None)
    ### Get the object in module
    attrs = qualname.split(".")
    obj = module
    last_inspectable_obj = obj
    last_inspectable_idx = 0
    for i in range(len(attrs)):
        attr = attrs[i]
        try:
            obj = getattr(obj, attr)
        except AttributeError:
            raise AttributeNameError(".".join([module.__name__]+attrs[:i]),
                                     attr)
        else:
            obj = getattr(obj, "__wrapped__", obj)
            if _is_inspectable(obj):
                last_inspectable_obj = obj
                last_inspectable_idx = i
    last_inspectable_obj_qualname = ".".join(attrs[:last_inspectable_idx+1])
    ### Get location
    last_inspectable_locs = _get_locations(last_inspectable_obj,
                                           last_inspectable_obj_qualname)
    if last_inspectable_obj == obj:
        return last_inspectable_locs
    ### Further investigate location of non-inspect-able object.
    assert _has_same_filename(last_inspectable_locs)
    filename = last_inspectable_locs[0].filename
    candidates = _search_assign(filename, qualname)
    if candidates:
        return _candidate_nodes_to_locations(filename, candidates)
    return [Location(filename, None, None)]

# =============================== #
# Command line interface function #
# =============================== #

DEFAULT_LOC_FORMAT = "emacs"

def format_loc(loc, format=DEFAULT_LOC_FORMAT):
    if format == 'emacs' or format == 'vi':
        s = ""
        if loc.line:
            s += "+%d" %(loc.line,)
            if loc.column:
                s += ":%d " %(loc.column,)
            else:
                s += " "
        s += loc.filename
        return s
    elif format == 'human':
        s = "Filename: %s" %(loc.filename,)
        if loc.line:
            s += "\nLine: %d" %(loc.line,)
        if loc.column:
            s += "\nColumn: %d" %(loc.column,)
        return s
    else:
        raise ValueError("unsupported format: {}".format(format))

_EPILOGUE = """
environment variables:
 PYLOC_DEFAULT_FORMAT - default output format (default: {default_format})

Copyright (c) 2015-2016, Nicolas Despres
All right reserved.
""".format(
    default_format=DEFAULT_LOC_FORMAT,
    )

def _build_cli():

    class LazyVersionAction(argparse.Action):
        """Replacement for the default 'version' action.

        This action is a lazily evaluate the 'version' keyword argument. The
        default 'version' action does not provide this feature.

        When called in develop mode some version string might be expensive
        to compute since they require to probe the underlying code repository.
        """

        def __init__(self, option_strings, dest, nargs=None, **kwargs):
            if nargs is not None:
                raise ValueError("'nargs' is not allowed.")
            if "choices" in kwargs:
                raise ValueError("'choices' is not allowed")
            if "type" in kwargs:
                raise ValueError("'type' is not allowed")
            self.version = kwargs.pop('version')
            super(LazyVersionAction, self).__init__(option_strings, dest,
                                                    nargs=0,
                                                    **kwargs)

        def __call__(self, parser, namespace, values, option_string=None):
            print(self.version())
            sys.exit(0)

    class RawDescriptionWithArgumentDefaultsHelpFormatter(
            argparse.ArgumentDefaultsHelpFormatter,
            argparse.RawDescriptionHelpFormatter,
    ):
        """Mix both formatter."""

    def _version():
        return \
            "pyloc {v} "\
            "on python {pyv.major}.{pyv.minor}.{pyv.micro} "\
            "(rev: {rev})"\
            .format(v=get_version(),
                    pyv=sys.version_info,
                    rev=get_revision())

    parser = argparse.ArgumentParser(
        description=__doc__,
        epilog=dedent(_EPILOGUE),
        formatter_class=RawDescriptionWithArgumentDefaultsHelpFormatter)
    parser.add_argument(
        "-f", "--format",
        action="store",
        choices=("emacs", "vi", "human"),
        default=os.environ.get("PYLOC_DEFAULT_FORMAT", DEFAULT_LOC_FORMAT),
        help="How to write object location")
    parser.add_argument(
        "-a", "--all",
        action="store_true",
        help="Print all possible location in case ambiguities")
    parser.add_argument(
        "--version",
        action=LazyVersionAction,
        version=_version)
    parser.add_argument(
        "object_name",
        action="store",
        help="A python object named: module[:qualname]")
    return parser

def _error(msg):
    sys.stderr.write("pyloc: ")
    sys.stderr.write(msg)
    sys.stderr.write("\n")

def _main():
    cli = _build_cli()
    options = cli.parse_args(sys.argv[1:])
    try:
        locs = pyloc(options.object_name)
    except PylocError as e:
        _error(str(e))
        return 1
    else:
        if options.all:
            locs_to_print = locs
        else:
            if len(locs) > 1:
                assert _has_same_filename(locs)
                locs_to_print = [locs[0]]
            else:
                locs_to_print = locs
        for loc in locs_to_print:
            sys.stdout.write(format_loc(loc, format=options.format))
            sys.stdout.write("\n")
        return 0

if __name__ == "__main__":
    sys.exit(_main())



# -*- encoding: utf-8 -*-
"""Test 'pyloc' module.

Copyright (c) 2015-2016, Nicolas Desprès
"""


import unittest
import sysconfig
import sys
import os
import contextlib
import re
import textwrap
import tempfile
import shutil

from pyloc import pyloc
from pyloc import ModuleNameError
from pyloc import AttributeNameError

# Guidelines:
# - Generate the package/module fixture for testing.
# - If you cannot, use only fixture names that are part of the standard
#   library for all supported version of Python and that are not imported
#   by either this module and pyloc.


PY_VERSION = tuple(map(int, sysconfig.get_config_var('py_version').split(".")))

def none_or_int(v):
    if v is None:
        return None
    else:
        return int(v)

def close_if_not_none(o):
    if o is not None:
        o.close()

@contextlib.contextmanager
def save_sys_modules():
    saved_modules = sys.modules.copy()
    try:
        yield
    finally:
        to_del = []
        for k in sys.modules:
            if k not in saved_modules:
                to_del.append(k)
        for k in to_del:
            del sys.modules[k]

def gen_fixture_in(spec, dirpath):
    for k in spec:
        v = spec[k]
        subpath = os.path.join(dirpath, k)
        if isinstance(v, dict):
            os.mkdir(subpath)
            with open(os.path.join(subpath, "__init__.py"), "w") as stream:
                stream.write("# -*- encoding: utf-8 -*-\n")
                stream.write("\"\"\"generated package\n\"\"\"")
            gen_fixture_in(v, subpath)
        else:
            with open(subpath+".py", "w") as stream:
                stream.write(v)

@contextlib.contextmanager
def make_tmpdir(*args, **kwargs):
    """Make temporary directory.

    Needed because tempfile.TemporaryDirectory is Python3 only.
    """
    try:
        tmpdir = tempfile.mkdtemp(*args, **kwargs)
        yield tmpdir
    finally:
        shutil.rmtree(tmpdir)

class FixtureCtxt(object):

    def __init__(self, testcase, tmpdir):
        self.testcase = testcase
        self.tmpdir = tmpdir

    def assertLocEqual(self, *args, **kwargs):
        self.testcase.assertLocEqual(self.tmpdir, *args, **kwargs)

class CompatAssert(object):
    """Collection of assert method compatible across various python version.
    """

    # Duplicated from unittest.TestCase because it is named assertRegexpMatches
    # in python 2.7.x
    def assertRegexp(self, text, expected_regex, msg=None):
        if isinstance(expected_regex, (str, bytes)):
            assert expected_regex, "expected_regex must not be empty."
            expected_regex = re.compile(expected_regex)
        if not expected_regex.search(text):
            msg = msg or "Regex didn't match"
            msg = '%s: %r not found in %r' % (msg, expected_regex.pattern, text)
            raise self.failureException(msg)

class TestPyloc(unittest.TestCase, CompatAssert):

    def assertLocEqual(self, rootdir, expected, modname, qualname=None,
                       locs=None, sep=":"):
        self.assertFalse(modname in sys.modules,
                         "'%s' is already imported" % (modname,))
        fullname = modname
        if qualname:
            fullname += sep + qualname
        if not os.path.isabs(expected):
            expected = os.path.join(rootdir, expected)
        if locs is None:
            locs = [(None, None)]
        elif isinstance(locs, tuple):
            locs = [locs]
        elif isinstance(locs, int):
            locs = [(locs, None)]
        with save_sys_modules():
            actuals = pyloc(fullname)
        self.assertEqual(len(locs), len(actuals))
        for actual, loc in zip(actuals, locs):
            self.assertEqual(expected, actual.filename)
            self.assertEqual(loc[0], actual.line)
            self.assertEqual(loc[1], actual.column)

    @contextlib.contextmanager
    def fixture(self, spec):
        with make_tmpdir() as tmpdir, \
             save_sys_modules():
            sys.path.insert(0, tmpdir)
            gen_fixture_in(spec, tmpdir)
            yield FixtureCtxt(self, tmpdir)

    def test_module_name_error(self):
        with self.assertRaises(ModuleNameError):
            pyloc("list")

    def test_report_exception_raised_from_imported_module(self):
        modcontent = textwrap.dedent(
            """\
            raise RuntimeError("intentional error")
            """)
        with self.fixture({"pyloc_mymod":modcontent}):
            with self.assertRaises(RuntimeError) as cm:
                pyloc("pyloc_mymod")
            self.assertRegexp(str(cm.exception), "intentional error")

    def test_attribute_name_error(self):
        with self.assertRaises(AttributeNameError) as cm:
            pyloc("subprocess:doesnotexist")
        self.assertRegexp(str(cm.exception),
                          r"'doesnotexist'.+'subprocess'")

    def test_sub_attribute_name_error(self):
        with self.assertRaises(AttributeNameError) as cm:
            pyloc("subprocess:Popen.doesnotexist")
        self.assertRegexp(str(cm.exception),
                          r"'doesnotexist'.+'subprocess.Popen'")


    def test_package(self):
        spec = {"pyloc_testpkg":{"utils":"def func(): pass"}}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/__init__.py", "pyloc_testpkg")

    def test_no_source_file(self):
        with save_sys_modules():
            import mmap
            filename = mmap.__file__
        self.assertLocEqual(None, # Ignored because filename is absolute
                            filename, "mmap")

    def test_function_in_native_module(self):
        with save_sys_modules():
            import mmap
            filename = mmap.__file__
        self.assertLocEqual(None, # Ignored because filename is absolute
                            filename, "mmap", qualname="mmap")

    def test_module(self):
        with self.fixture({"pyloc_testmod":""}) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod")

    def test_builtin_module(self):
        with self.assertRaises(TypeError) as cm:
            pyloc("sys")
        self.assertRegexp(str(cm.exception), r"is a built-in module")

    def test_builtin_module_in_class(self):
        modcontent = textwrap.dedent(
            """\
            class C(object):
                import sys
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            with self.assertRaises(TypeError) as cm:
                pyloc("pyloc_testmod:C.sys")
            self.assertRegexp(str(cm.exception), r"is a built-in module")

    def test_module_in_package(self):
        spec = {"pyloc_testpkg":{"utils":""}}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/utils.py",
                                 "pyloc_testpkg.utils")

    def test_module_in_module(self):
        spec = {"pyloc_testmod":"import os"}
        with self.fixture(spec) as fctxt:
            self.assertLocEqual(None, os.__file__.rstrip("c"),
                                "pyloc_testmod", qualname="os")

    def test_module_in_class(self):
        modcontent = textwrap.dedent(
            """\
            class C(object):
                import os
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            self.assertLocEqual(None, os.__file__.rstrip("c"),
                                "pyloc_testmod", qualname="C.os")

    def test_follow_imported_module(self):
        spec = {
            "pyloc_testpkg": {
                "mod1": textwrap.dedent(
                    """\
                    def realf():
                        pass
                    """),
                "mod2": textwrap.dedent(
                    """\
                    from pyloc_testpkg import mod1
                    """),
            },
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/mod1.py", "pyloc_testpkg.mod2",
                                 qualname="mod1")

    def test_follow_imported_module_as(self):
        spec = {
            "pyloc_testpkg": {
                "mod1": textwrap.dedent(
                    """\
                    def realf():
                        pass
                    """),
                "mod2": textwrap.dedent(
                    """\
                    from pyloc_testpkg import mod1 as m
                    """),
            },
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/mod1.py", "pyloc_testpkg.mod2",
                                 qualname="m")

    def test_follow_imported_module_star(self):
        spec = {
            "pyloc_testpkg": {
                "subpkg": {
                    "__init__": textwrap.dedent(
                    """\
                    __all__ = ["mod1", "mod2"]
                    """),
                    "mod1": "",
                    "mod2": "",
                },
                "mod": textwrap.dedent(
                    """\
                    from pyloc_testpkg.subpkg import *
                    """),
            },
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/subpkg/mod1.py",
                                 "pyloc_testpkg.mod", qualname="mod1")

    def test_function_in_module_in_package(self):
        spec = {"pyloc_testpkg":{"utils":"def func(): pass"}}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/utils.py",
                                 "pyloc_testpkg.utils",
                                 qualname="func",
                                 locs=1)

    def test_function_in_module(self):
        spec = {"pyloc_testmod":"def func(): pass"}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="func",
                                 locs=1)

    def test_multiple_functions(self):
        for cond, loc in ((True, 3), (False, 6)):
            modcontent = textwrap.dedent(
                """\
                cond = {cond}
                if cond:
                    def f():
                        pass
                else:
                    def f():
                        pass
                """.format(cond=cond))
            spec = {"pyloc_testmod":modcontent}
            with self.fixture(spec) as fctxt:
                fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                     qualname="f",
                                     locs=loc)

    def test_follow_aliases(self):
        modcontent = textwrap.dedent(
            """\
            def realf():
                pass
            f = realf
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="f",
                                 locs=1)

    def test_follow_imported_function(self):
        spec = {
            "pyloc_testpkg": {
                "mod1": textwrap.dedent(
                    """\
                    def realf():
                        pass
                    """),
                "mod2": textwrap.dedent(
                    """\
                    from pyloc_testpkg.mod1 import realf
                    """),
            },
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/mod1.py", "pyloc_testpkg.mod2",
                                 qualname="realf",
                                 locs=1)

    def test_follow_imported_function_as(self):
        spec = {
            "pyloc_testpkg": {
                "mod1": textwrap.dedent(
                    """\
                    def realf():
                        pass
                    """),
                "mod2": textwrap.dedent(
                    """\
                    from pyloc_testpkg.mod1 import realf as f
                    """),
            },
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/mod1.py", "pyloc_testpkg.mod2",
                                 qualname="f",
                                 locs=1)

    def test_follow_imported_function_star(self):
        spec = {
            "pyloc_testpkg": {
                "mod1": textwrap.dedent(
                    """\
                    def realf():
                        pass
                    """),
                "mod2": textwrap.dedent(
                    """\
                    from pyloc_testpkg.mod1 import *
                    """),
            },
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/mod1.py", "pyloc_testpkg.mod2",
                                 qualname="realf",
                                 locs=1)

    def test_closure(self):
        modcontent = textwrap.dedent(
            """\
            def realf():
                def f():
                    pass
                return f
            func = realf()
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="func",
                                 locs=2)

    def test_setattr(self):
        modcontent = textwrap.dedent(
            """\
            import sys
            def realf():
                def f():
                    pass
                return f
            this_module = sys.modules[__name__]
            setattr(this_module, "func", realf())
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="func",
                                 locs=3)

    def test_class(self):
        spec = {"pyloc_testmod":"class Foo(object): pass"}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="Foo",
                                 locs=(1, 0))

    def test_multiple_classes(self):
        for cond in (True, False):
            modcontent = textwrap.dedent(
                """\
                cond = {cond}
                if cond:
                    class C(object):
                        pass
                else:
                    class C(object):
                        pass
                """.format(cond=cond))
            spec = {"pyloc_testmod":modcontent}
            with self.fixture(spec) as fctxt:
                fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                     qualname="C",
                                     locs=[(3, 4), (6, 4)])

    def test_multiple_classes_disamb(self):
        for cond, loc in ((True, (3, 4)), (False, (7, 4))):
            modcontent = textwrap.dedent(
                """\
                cond = {cond}
                if cond:
                    class C(object):
                        def m():
                            pass
                else:
                    class C(object):
                        def m():
                            pass
                """.format(cond=cond))
            spec = {"pyloc_testmod":modcontent}
            with self.fixture(spec) as fctxt:
                fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                     qualname="C",
                                     locs=loc)

    def test_class_robust_comment(self):
        modcontent = textwrap.dedent(
            """\
            # -*- encoding: utf-8 -*-
            # This module defines the class called Foo:
            #
            # class Foo()
            #

            class Foo(object):
                pass
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="Foo",
                                 locs=(7, 0))

    def test_class_robust_docstring(self):
        modcontent = textwrap.dedent(
            """\
            # -*- encoding: utf-8 -*-
            \"\"\"
            This module defines the class called Foo:

            class Foo()
            \"\"\"

            class Foo(object):
                pass
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="Foo",
                                 locs=(8, 0))

    def test_class_robust_fundef(self):
        modcontent = textwrap.dedent(
            """\
            def f():
                class C(object):
                    pass
            class C(object):
                pass
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="C",
                                 locs=(4, 0))

    def test_nested_class(self):
        modcontent = textwrap.dedent(
            """\
            # -*- encoding: utf-8 -*-
            class Foo(object):
                class Foo(object):
                    class Foo(object):
                        pass
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="Foo.Foo.Foo",
                                 locs=(4, 8))

    def test_class_namedtuple(self):
        modcontent = textwrap.dedent(
            """\
            from collections import namedtuple
            Point = namedtuple("Point", "x y")
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="Point",
                                 locs=(2, 0))

    def test_class_nested_namedtuple(self):
        modcontent = textwrap.dedent(
            """\
            from collections import namedtuple
            class C(object):
                Point = namedtuple("Point", "x y")
            Point = C.Point(1, 2)
            def f():
                Point = C.Point(2, 3)
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="C.Point",
                                 locs=(3, 4))

    def test_class_nested_namedtuple(self):
        modcontent = textwrap.dedent(
            """\
            from collections import namedtuple
            class C(object):
                Point = namedtuple("Point", "x y")
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="C.Point",
                                 locs=(3, 4))

    def test_class_alias(self):
        modcontent = textwrap.dedent(
            """\
            def f():
                D = 51
            class C(object):
                D = 42
            D = C
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="D",
                                 locs=(5, 0))

    def test_class_alias_redefinition(self):
        modcontent = textwrap.dedent(
            """\
            class C(object):
                D = 42
            D = 42
            D = C
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="D",
                                 locs=[(3, 0),(4,0)])

    def test_class_multiple_assignment(self):
        modcontent = textwrap.dedent(
            """\
            class C(object):
                D = 42
            _, D = (1, C)
            """)
        spec = {"pyloc_testmod":modcontent}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="D",
                                 locs=(3, 0))

    def test_follow_imported_class(self):
        spec = {
            "pyloc_testpkg": {
                "mod1": textwrap.dedent(
                    """\
                    class RealC(object):
                        pass
                    """),
                "mod2": textwrap.dedent(
                    """\

                    from pyloc_testpkg.mod1 import RealC
                    """),
            },
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/mod1.py", "pyloc_testpkg.mod2",
                                 qualname="RealC",
                                 locs=(1, 0))

    def test_follow_imported_class_as(self):
        spec = {
            "pyloc_testpkg": {
                "mod1": textwrap.dedent(
                    """\
                    class RealC(object):
                        pass
                    """),
                "mod2": textwrap.dedent(
                    """\
                    from pyloc_testpkg.mod1 import RealC as C
                    """),
            },
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/mod1.py", "pyloc_testpkg.mod2",
                                 qualname="C",
                                 # Cannot get line/column because C does not
                                 # appear in mod1.py.
                                 locs=None)

    def test_follow_imported_class_star(self):
        spec = {
            "pyloc_testpkg": {
                "mod1": textwrap.dedent(
                    """\
                    class RealC(object):
                        pass
                    """),
                "mod2": textwrap.dedent(
                    """\
                    from pyloc_testpkg.mod1 import *
                    """),
            },
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/mod1.py", "pyloc_testpkg.mod2",
                                 qualname="RealC",
                                 locs=(1, 0))

    def test_method(self):
        modcontent = textwrap.dedent(
            """\
            class Foo(object):
                def meth(self):
                    pass
            """)
        with self.fixture({"pyloc_testmod":modcontent}) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="Foo.meth",
                                 locs=2)

    def test_nested_method(self):
        modcontent = textwrap.dedent(
            """\
            def meth():
                pass
            class Bar(object):
                def meth(self):
                    pass
            class Foo(object):
                def meth(self):
                    pass
            class A(object):
                class B(object):
                    def meth(self):
                        pass
            """)
        with self.fixture({"pyloc_testmod":modcontent}) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="Foo.meth",
                                 locs=7)
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="Bar.meth",
                                 locs=4)
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="A.B.meth",
                                 locs=11)
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="meth",
                                 locs=1)

    def test_constant(self):
        modcontent = textwrap.dedent(
            """\
            class C(object):
                PI = 3.14
            class D(object):
                _, PI = (2, 3.14)
            PI = 3.14
            """)
        with self.fixture({"pyloc_testmod":modcontent}) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="PI", locs=(5, 0))
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="C.PI", locs=(2, 4))
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="D.PI", locs=(4, 4))

    def test_do_not_follow_imported_constant(self):
        spec = {
            "pyloc_testpkg": {
                "mod1": textwrap.dedent(
                    """\
                    PI = 3.14
                    """),
                "mod2": textwrap.dedent(
                    """\
                    from pyloc_testpkg.mod1 import PI
                    """),
            },
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/mod2.py", "pyloc_testpkg.mod2",
                                 qualname="PI", locs=(1, 0))

    def test_do_not_follow_imported_constant_as(self):
        spec = {
            "pyloc_testpkg": {
                "mod1": textwrap.dedent(
                    """\
                    PI = 3.14
                    """),
                "mod2": textwrap.dedent(
                    """\
                    from pyloc_testpkg.mod1 import PI as pi
                    """),
            },
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/mod2.py", "pyloc_testpkg.mod2",
                                 qualname="pi", locs=(1, 0))

    @unittest.skipIf(PY_VERSION >= (3, 0, 0), "test for python 2 only")
    def test_unicode(self):
        modcontent = textwrap.dedent(
            """\
            PI = 3.14
            """)
        with self.fixture({"pyloc_testmod":modcontent}) as fctxt:
            fctxt.assertLocEqual(unicode("pyloc_testmod.py"),
                                 unicode("pyloc_testmod"),
                                 qualname=unicode("PI"),
                                 locs=(1,0))

    @unittest.skipIf(PY_VERSION < (3, 2, 0), "__wrapped__ is set since 3.2")
    def test_follow_decorated_name(self):
        modcontent = textwrap.dedent(
            """\
            from contextlib import contextmanager
            @contextmanager
            def meth():
                pass
            class A:
                @contextmanager
                def meth(self):
                    pass
            """)
        with self.fixture({"pyloc_testmod":modcontent}) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="meth",
                                 locs=2)
            fctxt.assertLocEqual("pyloc_testmod.py", "pyloc_testmod",
                                 qualname="A.meth",
                                 locs=6)

    @unittest.skipIf(PY_VERSION < (3, 4, 0), "source_from_cache add since 3.4")
    def test_function_in_frozen_module(self):
        with save_sys_modules():
            locs = pyloc("importlib.util:source_from_cache")
            self.assertTrue(os.path.exists(locs[0].filename))

    def test_dot_package(self):
        spec = {"pyloc_testpkg":{"utils":"def func(): pass"}}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/utils.py",
                                 "pyloc_testpkg.utils",
                                 qualname="func", locs=1,
                                 sep=".")

    def test_dot_package_with_class(self):
        spec = {
            "pyloc_testpkg": {
                "mod1": textwrap.dedent(
                    """\
                    class C:
                        def func():
                            pass
                    """),
            },
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testpkg/mod1.py",
                                 "pyloc_testpkg.mod1",
                                 qualname="C.func", locs=2,
                                 sep=".")

    def test_dot_mod(self):
        spec = {"pyloc_testmod":"def func(): pass"}
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py",
                                 "pyloc_testmod",
                                 qualname="func", locs=1,
                                 sep=".")

    def test_dot_mod_with_class(self):
        spec = {
            "pyloc_testmod": textwrap.dedent(
                """\
                class C:
                    def func():
                        pass
                """),
        }
        with self.fixture(spec) as fctxt:
            fctxt.assertLocEqual("pyloc_testmod.py",
                                 "pyloc_testmod",
                                 qualname="C.func", locs=2,
                                 sep=".")

class TestCLI(unittest.TestCase):
    """Base class of command line interface test case.

    All test cases related to CLI should inherit from this class.
    """

    def setUp(self):
        super(TestCLI, self).setUp()
        self.pyloc = None
        self.tmpdir = os.path.realpath(tempfile.mkdtemp())

    def tearDown(self):
        if self.pyloc is not None:
            close_if_not_none(self.pyloc.stdout)
            close_if_not_none(self.pyloc.stderr)
        shutil.rmtree(self.tmpdir)
        super(TestCLI, self).tearDown()

    def run_pyloc(self, *args, **kwargs):
        # Import it from here because it is used as a fixture by other test
        # case.
        import subprocess as sp
        exe = sys.executable
        cmd = [exe, '-m', 'pyloc']
        cmd.extend(args)
        env = kwargs.get("env")
        if env is None:
            env = os.environ.copy()
        pythonpath = kwargs.get("pythonpath")
        if pythonpath is not None:
            env.setdefault("PYTHONPATH", "")
            env["PYTHONPATH"] += ":"+":".join(pythonpath)
        self.pyloc = sp.Popen(cmd,
                              executable=exe,
                              stdout=sp.PIPE,
                              stderr=sp.PIPE,
                              universal_newlines=True,
                              env=env)
        self.pyloc.wait()
        return self.pyloc.returncode

    def gen_fixture(self, spec):
        gen_fixture_in(spec, self.tmpdir)

class CLITestMethods(CompatAssert):
    """Collection of test method useful for any format."""

    def test_format(self):
        self.gen_fixture({"testmod":""})
        self.assertEqual(self.run_pyloc('--format', self.FORMAT,
                                        'testmod', pythonpath=[self.tmpdir]), 0)
        self.assertOutput(self.pyloc.stdout.read(),
                          [(os.path.join(self.tmpdir, "testmod.py"),
                            None,
                            None)])

    def test_format_multi_with_all(self):
        modcontent = textwrap.dedent(
            """\
            cond = True
            if cond:
                class C(object):
                    pass
            else:
                class C(object):
                    pass
            """)
        spec = {"pyloc_testmod":modcontent}
        self.gen_fixture(spec)
        pyloc_rc = self.run_pyloc('--format', self.FORMAT, '--all',
                                  'pyloc_testmod.C',
                                  pythonpath=[self.tmpdir])
        self.assertMultiLineEqual("", self.pyloc.stderr.read())
        self.assertEqual(pyloc_rc, 0)
        modpathname = os.path.join(self.tmpdir, "pyloc_testmod.py")
        pyloc_stdout = self.pyloc.stdout.read()
        self.assertOutput(pyloc_stdout,
                          [(modpathname, 3, 4),
                           (modpathname, 6, 4)])

    def test_format_multi_without_all(self):
        modcontent = textwrap.dedent(
            """\
            cond = True
            if cond:
                class C(object):
                    pass
            else:
                class C(object):
                    pass
            """)
        spec = {"pyloc_testmod":modcontent}
        self.gen_fixture(spec)
        pyloc_rc = self.run_pyloc('--format', self.FORMAT,
                                  'pyloc_testmod.C',
                                  pythonpath=[self.tmpdir])
        self.assertMultiLineEqual("", self.pyloc.stderr.read())
        self.assertEqual(pyloc_rc, 0)
        modpathname = os.path.join(self.tmpdir, "pyloc_testmod.py")
        pyloc_stdout = self.pyloc.stdout.read()
        self.assertOutput(pyloc_stdout,
                          [(modpathname, 3, 4)])

    def test_unknown_root_module(self):
        pyloc_rc = self.run_pyloc('--format', self.FORMAT,
                                  'doesnotexist')
        self.assertRegexp(self.pyloc.stderr.read(),
                         r"^pyloc: failed to import 'doesnotexist' ")
        self.assertEqual(pyloc_rc, 1)
        self.assertMultiLineEqual("", self.pyloc.stdout.read())

    def test_unknown_module_in_package(self):
        spec = {"pyloc_testpkg":{"utils":"def func(): pass"}}
        self.gen_fixture(spec)
        pyloc_rc = self.run_pyloc('--format', self.FORMAT,
                                  'pyloc_testpkg.doesnotexist',
                                  pythonpath=[self.tmpdir])
        self.assertRegexp(self.pyloc.stderr.read(),
                         r"^pyloc: cannot get attribute 'doesnotexist' "\
                         "from 'pyloc_testpkg'")
        self.assertEqual(pyloc_rc, 1)
        self.assertMultiLineEqual("", self.pyloc.stdout.read())

class TestCLIHuman(TestCLI, CLITestMethods):

    FORMAT = 'human'

    def assertOutput(self, output, locs):
        iloc = 0
        lines = output.splitlines()
        # We need a dictionary because python 2.7 does not have 'nonlocal'
        # keyword
        d = {'iline': 0 }
        def _nextline():
            try:
                line = lines[d['iline']]
            except IndexError:
                return None
            d['iline'] += 1
            return line.rstrip("\n")
        while d['iline'] < len(lines):
            line = _nextline()
            mo = re.match(r"^Filename: (.+)$", line)
            if mo is None:
                self.fail("missing filename in human output")
            filename = mo.group(1)
            line_no = None
            line = _nextline()
            if line is not None:
                mo = re.match(r"^Line: (\d+)$", line)
                if mo is not None:
                    line_no = int(mo.group(1))
            col_no = None
            line = _nextline()
            if line is not None:
                mo = re.match(r"^Column: (\d+)$", line)
                if mo is not None:
                    col_no = int(mo.group(1))
            actual_loc = (filename, line_no, col_no)
            try:
                expected_loc = locs[iloc]
            except IndexError:
                self.fail("expected only %d locations, "
                          "missing %dth location: %r"
                          %(len(locs), iloc, actual_loc))
            else:
                self.assertTupleEqual(expected_loc, actual_loc)
            iloc += 1

class TestCLIEmacs(TestCLI, CLITestMethods):

    FORMAT = 'emacs'

    def assertOutput(self, output, locs):
        for iloc, line in enumerate(output.splitlines()):
            line = line.rstrip("\n")
            mo = re.match(
                r"^(\+(?P<line>\d+)(:(?P<col>\d+))? )?(?P<filename>.+)$",
                line)
            if mo is None:
                self.fail("missing filename in human output")
            filename = mo.group("filename")
            line_no = none_or_int(mo.group("line"))
            col_no = none_or_int(mo.group("col"))
            actual_loc = (filename, line_no, col_no)
            try:
                expected_loc = locs[iloc]
            except IndexError:
                self.fail("expected only %d locations, "
                          "missing %dth location: %r"
                          %(len(locs), iloc, actual_loc))
            else:
                self.assertTupleEqual(expected_loc, actual_loc)

class TestCLIVi(TestCLIEmacs):

    FORMAT = 'vi'

class TestVersion(unittest.TestCase):

    def setUp(self):
        super(TestVersion, self).setUp()

    def tearDown(self):
        super(TestVersion, self).tearDown()

    def test_version(self):
        # Import it from here because it is used as a fixture by other test
        # case.
        import subprocess as sp
        exe = sys.executable
        cmd = [exe,
               os.path.join(os.path.dirname(__file__), "script", "version"),
               "check"]
        proc = sp.Popen(cmd,
                        executable=exe,
                        stdout=sp.PIPE,
                        stderr=sp.STDOUT,
                        universal_newlines=True)
        proc.wait()
        for line in proc.stdout:
            sys.stdout.write(line)
        self.assertEqual(proc.returncode, 0)



# -*- encoding: utf-8 -*-
# Copyright (c) 2015-2016, Nicolas Desprès

# Relevant documentation used when writing this file:
#   https://docs.python.org/3/library/distutils.html
#   http://www.diveinto.org/python3/packaging.html
#   http://www.scotttorborg.com/python-packaging/
# and of course several example projects such as: csvkit, nose or buildout.

from setuptools import setup
from setuptools.command.sdist import sdist
from wheel.bdist_wheel import bdist_wheel

import os
import sys
import subprocess
import errno
import codecs
from contextlib import contextmanager

ROOT_DIR = os.path.dirname(__file__)
VERSION_TXT = os.path.join(ROOT_DIR, "VERSION.txt")
REVISION_TXT = os.path.join(ROOT_DIR, "REVISION.txt")
README_RST = os.path.join(ROOT_DIR, "README.rst")
VERSION_SCRIPT = os.path.join("script", "version")

def readfile(filepath):
    with codecs.open(filepath,
                     mode="r",
                     encoding="utf-8") as stream:
        return stream.read()

def writefile(filepath, content):
    with codecs.open(filepath, mode="w", encoding="utf-8") as stream:
        stream.write(content)

def get_version():
    try:
        return readfile(VERSION_TXT).strip()
    except IOError as e:
        if e.errno == errno.ENOENT:
            cmd = [VERSION_SCRIPT, "get", "--no-dirty"]
            return subprocess.check_output(cmd).decode().strip()

def get_revision():
    try:
        return readfile(REVISION_TXT).strip()
    except IOError as e:
        if e.errno == errno.ENOENT:
            cmd = [VERSION_SCRIPT, "revision"]
            return subprocess.check_output(cmd).decode().strip()

def is_git_dir(dirpath=None):
    if dirpath is None:
        dirpath = os.getcwd()
    cmd = ["git", "-C", str(dirpath), "rev-parse", "--git-dir"]
    try:
        with open(os.devnull, "wb") as devnull:
            subprocess.check_call(cmd, stdout=devnull, stderr=devnull)
    except subprocess.CalledProcessError:
        return False
    return True

def git_file_is_modified(filepath):
    cmd = ["git", "status", "--porcelain", str(filepath)]
    output = subprocess.check_output(cmd).decode()
    return output != u''

def sed_i(script, *filepaths):
    cmd = [ "sed", "-i", '', "-e", script ] + list(filepaths)
    subprocess.check_call(cmd)

def rm_f(path):
    try:
        os.unlink(path)
    except OSError as e:
        if e.errno != errno.ENOENT:
            raise e

@contextmanager
def inject_version_info():
    pyloc_py = "pyloc.py"
    if not is_git_dir():
        yield
        return
    if git_file_is_modified(pyloc_py):
        raise RuntimeError("'%s' has un-commited changes"%(pyloc_py,))
    try:
        print("inject version number into module")
        VERSION = get_version()
        sed_i(
            "s/^__version__ = 'dev'$/__version__ = '%s'/"%(VERSION,),
            pyloc_py)
        writefile(VERSION_TXT, VERSION)
        print("inject revision number into module")
        REVISION = get_revision()
        writefile(REVISION_TXT, REVISION)
        sed_i(
            "s/^__revision__ = 'git'$/__revision__ = '%s'/"%(REVISION,),
            pyloc_py)
        yield
    finally:
        print("restore %s"%(pyloc_py,))
        subprocess.check_call(["git", "checkout", pyloc_py])
        print("remove %s"%(VERSION_TXT,))
        rm_f(VERSION_TXT)
        print("remove %s"%(REVISION_TXT,))
        rm_f(REVISION_TXT)

class SDistProxy(sdist):
    """Hook sdist command"""

    def run(self):
        with inject_version_info():
            # Super class is an old-style class, so we use old-style
            # "super" call.
            sdist.run(self)

class BDistWheelProxy(bdist_wheel):

    def run(self):
        with inject_version_info():
            # Super class is an old-style class, so we use old-style
            # "super" call.
            bdist_wheel.run(self)

PY_VERSION_SUFFIX = '%s' % (sys.version_info.major,)

setup(
    name="pyloc",
    version=get_version(),
    # We only have a single module to distribute
    packages=[],
    py_modules=[
        "pyloc",
        "test_pyloc",
    ],
    # We only depends on Python standard library.
    install_requires=[],
    # Generate a command line interface driver.
    entry_points={
        'console_scripts': [
            "pyloc=pyloc:_main",
            "pyloc%s=pyloc:_main" % (PY_VERSION_SUFFIX,),
        ],
    },
    # How to run the test suite.
    test_suite='nose.collector',
    tests_require=['nose'],
    # What it does, who wrote it and where to find it.
    description="Locate python object definition in your file-system",
    long_description=readfile(README_RST),
    author="Nicolas Despres",
    author_email='nicolas.despres@gmail.com',
    license="Simplified BSD",
    keywords='utility',
    url='https://github.com/nicolasdespres/pyloc',
    platforms=["any"],
    # Pick some from https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
    cmdclass={
        'sdist': SDistProxy,
        'bdist_wheel': BDistWheelProxy,
    },
)
